@@ -3,37 +3,74 @@
 This plugin provides advanced doctest support and enables the testing of .rst
 files.
 """
-import six
-
 import doctest
 import fnmatch
-import imp
 import os
 import re
 import sys
 import warnings
 
 import pytest
+import six
 
-from .output_checker import OutputChecker, FIX
+from pytest_doctestplus.utils import ModuleChecker
+from .output_checker import FIX, IGNORE_WARNINGS, OutputChecker, REMOTE_DATA
 
-comment_characters = {'txt': '#',
-                      'tex': '%',
-                      'rst': '\.\.'
-                      }
+try:
+    from textwrap import indent
+except ImportError:  # PY2
+    def indent(text, prefix):
+        return '\n'.join([prefix + line for line in text.splitlines()])
+
+comment_characters = {
+    '.txt': '#',
+    '.tex': '%',
+    '.rst': r'\.\.'
+}
+
+
+# For the IGNORE_WARNINGS option, we create a context manager that doesn't
+# require us to add any imports to the example list and contains everything
+# that is needed to silence warnings.
+
+IGNORE_WARNINGS_CONTEXT = """
+class _doctestplus_ignore_all_warnings(object):
+
+    def __init__(self):
+        import warnings
+        self._cw = warnings.catch_warnings()
+
+    def __enter__(self, *args, **kwargs):
+        result = self._cw.__enter__(*args, **kwargs)
+        import warnings
+        warnings.simplefilter('ignore')
+        return result
+
+    def __exit__(self, *args, **kwargs):
+        return self._cw.__exit__(*args, **kwargs)
+""".lstrip()
 
 
 # these pytest hooks allow us to mark tests and run the marked tests with
 # specific command line options.
 def pytest_addoption(parser):
-
     parser.addoption("--doctest-plus", action="store_true",
                      help="enable running doctests with additional "
-                     "features not found in the normal doctest "
-                     "plugin")
+                          "features not found in the normal doctest "
+                          "plugin")
 
     parser.addoption("--doctest-rst", action="store_true",
-                     help="enable running doctests in .rst documentation")
+                     help=(
+                         "Enable running doctests in .rst documentation. "
+                         "This is no longer recommended, use --doctest-glob instead."
+                     ))
+
+    parser.addoption("--text-file-format", action="store",
+                     help=(
+                        "Text file format for narrative documentation. "
+                        "Options accepted are 'txt', 'tex', and 'rst'. "
+                        "This is no longer recommended, use --doctest-glob instead."
+                     ))
 
     # Defaults to `atol` parameter from `numpy.allclose`.
     parser.addoption("--doctest-plus-atol", action="store",
@@ -45,17 +82,18 @@ def pytest_addoption(parser):
                      help="set the relative tolerance for float comparison",
                      default=1e-05)
 
-    parser.addoption("--text-file-format", action="store",
-                     help=("Text file format for narrative documentation. "
-                           "Options accepted are 'txt', 'tex', and 'rst'"))
+    parser.addoption("--doctest-only", action="store_true",
+                     help="Test only doctests. Implies usage of doctest-plus.")
 
-    parser.addini("text_file_format", "changing default format for docs")
+    parser.addini("text_file_format",
+                  "Default format for docs. "
+                  "This is no longer recommended, use --doctest-glob instead.")
 
     parser.addini("doctest_optionflags", "option flags for doctests",
                   type="args", default=["ELLIPSIS", "NORMALIZE_WHITESPACE"],)
 
     parser.addini("doctest_plus", "enable running doctests with additional "
-                  "features not found in the normal doctest plugin")
+                                  "features not found in the normal doctest plugin")
 
     parser.addini("doctest_norecursedirs",
                   "like the norecursedirs option but applies only to doctest "
@@ -73,6 +111,11 @@ def pytest_addoption(parser):
                   "set the relative tolerance for float comparison",
                   default=1e-05)
 
+    parser.addini('text_file_comment_chars',
+                  help='list of pairs in format file_extension=comment_chars, eg: .rst=..',
+                  type='linelist',
+                  default=[])
+
 
 def get_optionflags(parent):
     optionflags_str = parent.config.getini('doctest_optionflags')
@@ -83,23 +126,30 @@ def get_optionflags(parent):
 
 
 def pytest_configure(config):
+    doctest_plugin = config.pluginmanager.getplugin('doctest')
+    run_regular_doctest = config.option.doctestmodules and not config.option.doctest_plus
+    use_doctest_plus = config.getini('doctest_plus') or config.option.doctest_plus or config.option.doctest_only
+    if doctest_plugin is None or run_regular_doctest or not use_doctest_plus:
+        return
 
     # We monkey-patch in our replacement doctest OutputChecker.  Not
     # great, but there isn't really an API to replace the checker when
     # using doctest.testfile, unfortunately.
+    doctest.OutputChecker = OutputChecker
     OutputChecker.rtol = max(float(config.getini("doctest_plus_rtol")),
                              float(config.getoption("doctest_plus_rtol")))
     OutputChecker.atol = max(float(config.getini("doctest_plus_atol")),
                              float(config.getoption("doctest_plus_atol")))
-    doctest.OutputChecker = OutputChecker
 
-    REMOTE_DATA = doctest.register_optionflag('REMOTE_DATA')
+    use_rst = config.getini('doctest_rst') or config.option.doctest_rst
+    file_ext = config.option.text_file_format or config.getini('text_file_format') or 'rst'
+    if use_rst:
+        config.option.doctestglob.append('*.{}'.format(file_ext))
 
-    doctest_plugin = config.pluginmanager.getplugin('doctest')
-    run_regular_doctest = config.option.doctestmodules and not config.option.doctest_plus
-    if (doctest_plugin is None or run_regular_doctest or not
-            (config.getini('doctest_plus') or config.option.doctest_plus)):
-        return
+    # override default comment characters
+    ext_comment_pairs = [pair.split('=') for pair in config.getini('text_file_comment_chars')]
+    for ext, chars in ext_comment_pairs:
+        comment_characters[ext] = chars
 
     class DocTestModulePlus(doctest_plugin.DoctestModule):
         # pytest 2.4.0 defines "collect".  Prior to that, it defined
@@ -135,15 +185,30 @@ def pytest_configure(config):
             for test in finder.find(module):
                 if test.examples:  # skip empty doctests
                     if config.getoption('remote_data', 'none') != 'any':
+
+                        ignore_warnings_context_needed = False
+
                         for example in test.examples:
+
+                            # If warnings are to be ignored we need to catch them by
+                            # wrapping the source in a context manager.
+                            if example.options.get(IGNORE_WARNINGS, False):
+                                example.source = ("with _doctestplus_ignore_all_warnings():\n"
+                                                + indent(example.source, '    '))
+                                ignore_warnings_context_needed = True
+
                             if example.options.get(REMOTE_DATA):
                                 example.options[doctest.SKIP] = True
 
+                        # We insert the definition of the context manager to ignore
+                        # warnings at the start of the file if needed.
+                        if ignore_warnings_context_needed:
+                            test.examples.insert(0, doctest.Example(source=IGNORE_WARNINGS_CONTEXT, want=''))
+
                     yield doctest_plugin.DoctestItem(
                         test.name, self, runner, test)
 
     class DocTestTextfilePlus(doctest_plugin.DoctestItem, pytest.Module):
-
         # Some pytest plugins such as hypothesis try and access the 'obj'
         # attribute, and by default this returns an error for this class
         # so we override it here to avoid any issues.
@@ -157,10 +222,10 @@ def pytest_configure(config):
 
             options = get_optionflags(self) | FIX
 
-            failed, tot = doctest.testfile(
+            doctest.testfile(
                 str(self.fspath), module_relative=False,
                 optionflags=options, parser=DocTestParserPlus(),
-                extraglobs=dict(getfixture=fixture_request.getfuncargvalue),
+                extraglobs=dict(getfixture=fixture_request.getfixturevalue),
                 raise_on_error=True, verbose=False, encoding='utf-8')
 
         def reportinfo(self):
@@ -203,17 +268,18 @@ def pytest_configure(config):
             skip_next = False
             skip_all = False
 
-            file_format = config.option.text_file_format or config.getini('text_file_format')
-
-            if file_format in comment_characters:
-                comment_char = comment_characters[file_format]
-            else:
+            ext = os.path.splitext(name)[1] if name else '.rst'
+            if ext not in comment_characters:
                 warnings.warn("file format '{}' is not recognized, assuming "
                               "'{}' as the comment character."
-                              .format(file_format, comment_characters['rst']))
-                comment_char = comment_characters['rst']
+                              .format(ext, comment_characters['rst']))
+                ext = '.rst'
+            comment_char = comment_characters[ext]
+
+            ignore_warnings_context_needed = False
 
             for entry in result:
+
                 if isinstance(entry, six.string_types) and entry:
                     required = []
                     skip_next = False
@@ -255,30 +321,45 @@ def pytest_configure(config):
                         match = matches[0]
 
                     if match:
-                        required = re.split(r'\s*,?\s*', match.group(1))
+                        # 'a a' or 'a,a' or 'a, a'-> [a, a]
+                        required = re.split(r'\s*[,\s]\s*', match.group(1))
                 elif isinstance(entry, doctest.Example):
-                    if (skip_all or skip_next or
-                        not DocTestFinderPlus.check_required_modules(required)):
+
+                    # If warnings are to be ignored we need to catch them by
+                    # wrapping the source in a context manager.
+                    if entry.options.get(IGNORE_WARNINGS, False):
+                        entry.source = ("with _doctestplus_ignore_all_warnings():\n"
+                                        + indent(entry.source, '    '))
+                        ignore_warnings_context_needed = True
+
+                    has_required_modules = DocTestFinderPlus.check_required_modules(required)
+                    if skip_all or skip_next or not has_required_modules:
                         entry.options[doctest.SKIP] = True
 
-                    if (config.getoption('remote_data', 'none') != 'any' and
-                        entry.options.get(REMOTE_DATA)):
+                    if config.getoption('remote_data', 'none') != 'any' and entry.options.get(REMOTE_DATA):
                         entry.options[doctest.SKIP] = True
 
+            # We insert the definition of the context manager to ignore
+            # warnings at the start of the file if needed.
+            if ignore_warnings_context_needed:
+                result.insert(0, doctest.Example(source=IGNORE_WARNINGS_CONTEXT, want=''))
+
             return result
 
     config.pluginmanager.register(
-        DoctestPlus(DocTestModulePlus, DocTestTextfilePlus,
-                    config.getini('doctest_rst') or config.option.doctest_rst,
-                    config.option.text_file_format or config.getini('text_file_format')),
-        'doctestplus')
+        DoctestPlus(
+            DocTestModulePlus,
+            DocTestTextfilePlus,
+            config.option.doctestglob,
+        ),
+        'doctestplus',
+    )
     # Remove the doctest_plugin, or we'll end up testing the .rst files twice.
     config.pluginmanager.unregister(doctest_plugin)
 
 
 class DoctestPlus(object):
-    def __init__(self, doctest_module_item_cls, doctest_textfile_item_cls,
-                 run_rst_doctests, text_file_format):
+    def __init__(self, doctest_module_item_cls, doctest_textfile_item_cls, file_globs):
         """
         doctest_module_item_cls should be a class inheriting
         `pytest.doctest.DoctestItem` and `pytest.File`.  This class handles
@@ -289,24 +370,39 @@ class DoctestPlus(object):
         """
         self._doctest_module_item_cls = doctest_module_item_cls
         self._doctest_textfile_item_cls = doctest_textfile_item_cls
-        self._run_rst_doctests = run_rst_doctests
-        if text_file_format:
-            self._text_file_ext = '.{}'.format(text_file_format)
-        else:
-            self._text_file_ext = '.rst'
+        self._file_globs = file_globs
         # Directories to ignore when adding doctests
         self._ignore_paths = []
 
     def pytest_ignore_collect(self, path, config):
-        """Skip paths that match any of the doctest_norecursedirs patterns."""
-        collect_ignore = config._getconftest_pathlist("collect_ignore",
-                                                      path=path.dirpath())
+        """
+        Skip paths that match any of the doctest_norecursedirs patterns or
+        if doctest_only is True then skip all regular test files (eg test_*.py).
+        """
+        collect_ignore = config._getconftest_pathlist("collect_ignore", path=path.dirpath())
 
         # The collect_ignore conftest.py variable should cause all test
         # runners to ignore this file and all subfiles and subdirectories
         if collect_ignore is not None and path in collect_ignore:
             return True
 
+        if config.option.doctest_only:
+            for pattern in config.getini('python_files'):
+                if path.check(fnmatch=pattern):
+                    return True
+
+        def get_list_opt(name):
+            return getattr(config.option, name, None) or []
+
+        for ignore_path in get_list_opt('ignore'):
+            ignore_path = os.path.abspath(ignore_path)
+            if str(path).startswith(ignore_path):
+                return True
+
+        for pattern in get_list_opt('ignore_glob'):
+            if path.check(fnmatch=pattern):
+                return True
+
         for pattern in config.getini("doctest_norecursedirs"):
             if path.check(fnmatch=pattern):
                 # Apparently pytest_ignore_collect causes files not to be
@@ -358,7 +454,7 @@ class DoctestPlus(object):
 
             # Don't override the built-in doctest plugin
             return self._doctest_module_item_cls(path, parent)
-        elif self._run_rst_doctests and path.ext == self._text_file_ext:
+        elif any([path.check(fnmatch=pat) for pat in self._file_globs]):
             # Ignore generated .rst files
             parts = str(path).split(os.path.sep)
 
@@ -393,34 +489,43 @@ class DocTestFinderPlus(doctest.DocTestFinder):
 
     # Caches the results of import attempts
     _import_cache = {}
+    _module_checker = ModuleChecker()
 
     @classmethod
     def check_required_modules(cls, mods):
+        """Check that modules in `mods` list are available.
+
+        Parameters
+        ----------
+        mods : list of str
+            List of modules. Modules can have specified versions (eg 'numpy>=1.15')
+
+        Returns
+        -------
+        bool
+            True if all modules in list are available.
+        """
         for mod in mods:
             if mod in cls._import_cache:
                 if not cls._import_cache[mod]:
                     return False
-            try:
-                imp.find_module(mod)
-            except ImportError:
+
+            if cls._module_checker.check(mod):
+                cls._import_cache[mod] = True
+            else:
                 cls._import_cache[mod] = False
                 return False
-            else:
-                cls._import_cache[mod] = True
         return True
 
-    def find(self, obj, name=None, module=None, globs=None,
-             extraglobs=None):
-        tests = doctest.DocTestFinder.find(self, obj, name, module, globs,
-                                           extraglobs)
-        if (hasattr(obj, '__doctest_skip__') or
-                hasattr(obj, '__doctest_requires__')):
+    def find(self, obj, name=None, module=None, globs=None, extraglobs=None):
+        tests = doctest.DocTestFinder.find(self, obj, name, module, globs, extraglobs)
+        if hasattr(obj, '__doctest_skip__') or hasattr(obj, '__doctest_requires__'):
             if name is None and hasattr(obj, '__name__'):
                 name = obj.__name__
             else:
                 raise ValueError("DocTestFinder.find: name must be given "
-                                 "when obj.__name__ doesn't exist: {!r}".format(
-                        (type(obj),)))
+                                 "when obj.__name__ doesn't exist: {!r}"
+                                 .format((type(obj),)))
 
             def test_filter(test):
                 for pat in getattr(obj, '__doctest_skip__', []):
@@ -436,8 +541,7 @@ class DocTestFinderPlus(doctest.DocTestFinder):
                     if not isinstance(pats, tuple):
                         pats = (pats,)
                     for pat in pats:
-                        if not fnmatch.fnmatch(test.name,
-                                               '.'.join((name, pat))):
+                        if not fnmatch.fnmatch(test.name, '.'.join((name, pat))):
                             continue
                         if not self.check_required_modules(mods):
                             return False
